; JAWS scripts for SecureCRT
; Written by Harald.van.Breederode@gmail.com

; Version 1.0   8-dec-2004 Initial coding
; Version 1.1 12-mar-2005 Added SecureCRT Version 5 support
; Version 1.2 21-nov-2005 Bug Fixes
; Version 2.0 5-dec-2015 Updated for SecureCRT 7.3

include "HjConst.jsh" ; Load the default JAWS constants
include "ssh.jsm" ; Load our messages

;const cSecureCRTWindowClass=  "AfxFrameOrView80u" ; This is the Window class of SecureCRT
;const cSecureCRTWindowClass=  "AfxFrameOrView120u"

globals 
	handle	hMain, ; Handle of the terminal text pane
	int iSecCrtForTheFirstTime, ; Is this the first time our scripts are being loaded?
	int iSecCrtNewTextWillBeSpoken, ; Do we automaticly speak new text?
	int iWeHitTheTop, ; Did we hit the Top of the Terminal Screen?
	int iWeHitTheBottom, ; Did we hit the Bottom of the Terminal Screen?
	int iDelay

; This script is  called by JawsKey+Q
; This script announces the currently active JAWS settings
Script ScriptFilename ()
	ScriptAndAppNames ("SecureCRT scripts") ; Announce that the SecureCRT settings are active
EndScript

; This Function flushes new text without speaking it
Void Function FlushNewText()
		let iSecCrtNewTextWillBeSpoken = 0
		ProcessNewText () ; Force a call to NewTextEvent
	Pause ()
		let iSecCrtNewTextWillBeSpoken = 1
EndFunction

; This function is activated when SecureCRT becomes active. Either when the program starts or when it gets focus again.
Void Function AutoStartEvent ()
if (SubString(GetWindowClass (GetFocus ()),1,14) == "AfxFrameOrView") then
;SayString("Yes")
		let hMain = GetCurrentWindow ()
	EndIf
	If (iSecCrtForTheFirstTime == false)	then ; Is this our first time?
		SayMessage (OT_APP_START, msgSecureCRTTutor) ; Speak our welcome message
		let iSecCrtForTheFirstTime = true; We do this only once
		let iSecCrtNewTextWillBeSpoken = true; By default we speak new text that appears on the screen
		let iDelay = 1
	EndIf
	SaveCursor () ; Place the current active cursor on the cursor stack
	JAWSCursor (); Activate the JAWS cursor
	SetRestriction (RestrictWindow) ; Restrict the JawsCursor to the main window
	RestoreCursor () ; Re activate the previous cursor

	FlushNewText () ; Get rid of all new text

;	Saystring(GetAppTitle ()) ; Speak the WindowTitle so we know where we are
;PerformScript BrailleSet8PixelsPerSpace()
;SetJcfOption(OPT_PIXELS_PER_SPACE,8)
EndFunction

Void Function AutoFinishEvent ()
;	SetJcfOption(OPT_PIXELS_PER_SPACE,999)
EndFunction

; This function is called when the active cursor hits the top ot the window
Void Function TopEdgeEvent (handle hWnd)
; We handle this only if the JAWScursor is active
	If  !( IsJAWSCursor () ) then
		Return ; Nothing to do unless the JAWS cursor is active
	EndIf
; Did the event go off in SecureCRT?
	if hwnd == hMain then ; Yes
		let iWeHitTheTop = true
	EndIf
EndFunction

; This function is called when the active cursor hits the bottom of the window.
Void Function BottomEdgeEvent (handle hWnd)
; We handle this only if the JAWScursor is active
	If  !( IsJAWSCursor () ) then
		Return ; Nothing to do unless the JAWS cursor is active
	EndIf
; Did the event go off in SecureCRT?
	if hwnd == hMain then ; Yes
		let iWeHitTheBottom  = true
	EndIf
EndFunction

; This script is called by JAWSKey+X
; It is used to toggle the new text being spoken on or off.
Script ToggleSpeechOnOrOff ()
	If (iSecCrtNewTextWillBeSpoken == false) then ; Is it Off?
		let iSecCrtNewTextWillBeSpoken = true; Turn it On
		SayMessage (OT_STATUS,msgSpeechOn_L,msgSpeechOn_S) ; announce the new setting
	Else
		let iSecCrtNewTextWillBeSpoken = false; Turn if Off
		SayMessage (OT_STATUS,msgSpeechOff_L,msgSpeechOff_S) ; Announce the new setting
	EndIf
EndScript

; This is called by JAWSKey+H
; It places the hotkeys used by these scripts in a help window
Script HotKeyHelp ()
	If UserBufferIsActive () then ; If the help window is already active
		UserBufferDeactivate (); Remove it
	EndIf
	SayFormattedMessage (OT_USER_BUFFER, msgHotKeyHelp) ; Place our message in the help window and activate it
EndScript

; This is called by JAWSKey+F1
; It places some genaral help into the help window
Script ScreenSensitiveHelp ()
	If UserBufferIsActive () then ; If the help window is already active
		UserBufferDeactivate () ; remove it
	EndIf
	SayFormattedMessage (OT_USER_BUFFER, msgScreenSensitiveHelp) ; Place our help message in the help window and activate it
EndScript

; This script is called by JAWSKey+P
; It is used to change the JAWS punctuation setting
Script ChangePunctuation ()
	PerformScript SetPunctuationLevel () ; Call the script in the default script file
EndScript

; This script is called by Control+TAB and CTRL+SHIFT+TAB
; It is used to switch between SecureCRT sessions
Script ControlTab ()
	var
		string sCurrentTitle,
		string sNewTitle

	let SCurrentTitle = StringReplaceSubstrings (GetAppTitle () , "SecureCRT","")

	TypeCurrentScriptKey () ; Pass Control+TAB to SecureCRT
	FlushNewText () ; Get rid of all new text
	let SNewTitle = StringReplaceSubstrings (GetAppTitle () , "SecureCRT","")
	If (sNewTitle != sCurrentTitle ) then
;		SayString(sNewTitle)
	EndIf
EndScript

; This script is called by ALT+1
Script Alt1 ()
	TypeCurrentScriptKey () ; Pass ALT+1 to SecureCRT
	FlushNewText () ; Get rid of all new text
EndScript

; This script is called by ALT+2
Script Alt2 ()
	TypeCurrentScriptKey () ; Pass ALT+2 to SecureCRT
	FlushNewText () ; Get rid of all new text
EndScript

; This script is called by ALT+3
Script Alt3 ()
	TypeCurrentScriptKey () ; Pass ALT+3 to SecureCRT
	FlushNewText () ; Get rid of all new text
EndScript

; This script is called by ALT+4
Script Alt4 ()
	TypeCurrentScriptKey () ; Pass ALT+4 to SecureCRT
	FlushNewText () ; Get rid of all new text
EndScript

; This script is called by ALT+5
Script Alt5 ()
	TypeCurrentScriptKey () ; Pass ALT+5 to SecureCRT
	FlushNewText () ; Get rid of all new text
EndScript

; This script is called by ALT+6
Script Alt6 ()
	TypeCurrentScriptKey () ; Pass ALT+6 to SecureCRT
	FlushNewText () ; Get rid of all new text
EndScript

; This script is called by ALT+7
Script Alt7 ()
	TypeCurrentScriptKey () ; Pass ALT+7 to SecureCRT
	FlushNewText () ; Get rid of all new text
EndScript

; This script is called by ALT+8
Script Alt8 ()
	TypeCurrentScriptKey () ; Pass ALT+8 to SecureCRT
	FlushNewText () ; Get rid of all new text
EndScript

; This script is called by ALT+9
Script Alt9 ()
	TypeCurrentScriptKey () ; Pass ALT+9 to SecureCRT
	FlushNewText () ; Get rid of all new text
EndScript

; This script is called by ALT+0
Script Alt0 ()
	TypeCurrentScriptKey () ; Pass ALT+0 to SecureCRT
	FlushNewText () ; Get rid of all new text
EndScript

; This script is called by the rightArrowKey
Script RightArrow ()
;	If (GetWindowClass (GetFocus ()) == cSecureCRTWindowClass) then ; Are we in the terminal screen?
if (SubString(GetWindowClass (GetFocus ()),1,14) == "AfxFrameOrView") then
		If IsPCCursor () then ; And is the PC cursor active?
;			SpeechOff () ; turn the speech off
			TypeCurrentScriptKey () ; pass the key to SecureCRT
			FlushNewText () 
;			Pause () ; Let NewTextEvent do its job (silently)
;			SpeechOn () ; Turn the speech back on
			If iDelay then
			SpeechOff ()	
				Delay (iDelay, FALSE)
				SpeechOn ()
			EndIf
			SayCharacter () ; Speak the character we moved to
		Else ; For the other cursors
			PerformScript SayNextCharacter() ; Call the default script
		EndIf
	Else ; In case we were not in the terminal window
		PerformScript SayNextCharacter() ; call the default script
	EndIf
EndScript

; This script is called by the LeftArrowKey
Script LeftArrow ()
;	If (GetWindowClass (GetFocus ()) == cSecureCRTWindowClass) then ; Are we in the terminal window?
if (SubString(GetWindowClass (GetFocus ()),1,14) == "AfxFrameOrView") then
		If IsPCCursor () then ; And is the PC cursor active?
;		SpeechOff () ; Turn the speech off
			TypeCurrentScriptKey () ; pass the key to SecureCRT
			FlushNewText () 
;			Pause () ; let NewTextEvent do its job (silently)
;			SpeechOn () ; Turn the speech back on
			If iDelay then
			SpeechOff ()	
				Delay (iDelay, FALSE)
				SpeechOn ()
			EndIf
			SayCharacter () ; speak the character we moved to
		Else ; for the other cursors 
			PerformScript SayPriorCharacter() ; call the default script
		EndIf
	Else ; In case we were not in the terminal window
		PerformScript SayPriorCharacter() ; call the default script
	EndIf
EndScript

; This is call by the UpArrowKey
Script UpArrow ()
;	If (GetWindowClass (GetFocus ()) == cSecureCRTWindowClass) then ; Are we in the terminal window?
if (SubString(GetWindowClass (GetFocus ()),1,14) == "AfxFrameOrView") then
		If IsPCCursor () then ; And is the PC cursor active?
;		SpeechOff () ; Turn the speech off
			TypeCurrentScriptKey () ; pass the key to SecureCRT
;			Pause () ; let NewTextEvent do its job (silently)
;			SpeechOn () ; turn the speech back on
			FlushNewText ()
			If iDelay then
			SpeechOff ()	
				Delay (iDelay, FALSE)
				SpeechOn ()
			EndIf
			SayLine () ; speak the line we moved to
		ElIf IsJAWSCursor () then
			PriorLine ()
			If iWeHitTheTop then
				let iWeHitTheTop = false
				{Control+PageUp} ; scroll the screen one row down
				FlushNewText ()
			EndIf
			SayLine ()
		Else ; for the other cursors
			PerformScript SayPriorLine() ; call the default scripts
		EndIf
	Else ; in case we were not in the terminal window
	PerformScript SayPriorLine(); call the default script
	EndIf
EndScript

; This script is called by the DownArrowKey
Script DownArrow ()
;	If (GetWindowClass (GetFocus ()) == cSecureCRTWindowClass) then; Are we in the terminal window?
if (SubString(GetWindowClass (GetFocus ()),1,14) == "AfxFrameOrView") then
		If IsPCCursor () then ; And is the PC cursor active?
;		SpeechOff () ; Turn the speech off
			TypeCurrentScriptKey () ; pass the key to SecureCRT
			FlushNewText () 
;			Pause () ; let NewTextEvent do its job (silently)
;			SpeechOn () ; Turn the speech back on
			If iDelay then
			SpeechOff ()	
				Delay (iDelay, FALSE)
				SpeechOn ()
			EndIf
			SayLine () ; speak the line we moved to
		ElIf IsJAWSCursor () then
			NextLine ()
			If iWeHitTheBottom then
				let iWeHitTheBottom = false
				{Control+PageDown} ; scroll the screen one row down
				FlushNewText ()
			EndIf
			SayLine ()

		Else ; for the other cursors
			PerformScript SayNextLine() ; call the default script
		EndIf
	Else ; in case we were not in the terminal window 
		PerformScript SayNextLine() ; call the default script
	EndIf
EndScript

int Function HandleCustomWindows(handle hwnd)
;SayString("HandleCustomWindows")
;	If (GetWindowClass (hwnd) == cSecureCRTWindowClass) then ; Is this our terminal screen?
if (SubString(GetWindowClass (GetFocus ()),1,14) == "AfxFrameOrView") then
		let hMain = hwnd
	SayString(StringReplaceSubstrings (GetAppTitle () , "SecureCRT",""))
		return TRUE
	EndIf
	return FALSE
EndFunction

Void Function SayNonHighlightedText (handle hwnd, string buffer)
;SayString("SayNonHighlightedText ")
; Is the new text within SecureCRT?
	if hwnd == hMain then ; Yes
		If (iSecCrtNewTextWillBeSpoken) then ; Do wwe need to speak it?
			SayMessage (OT_SCREEN_MESSAGE,buffer) ; Yes
		EndIf
		Return ; We handled new SecureCRT text
	EndIf
; Everything we did not handled yet gets passed to the default function
	SayNonHighlightedText (hwnd, buffer)
EndFunction

Script IncreaseDelay ()
	Let iDelay = iDelay + 1
	SayInteger (iDelay)
EndScript

Script DecreaseDelay ()
	Let iDelay = iDelay - 1
	SayInteger (iDelay)
EndScript
